@@ -12,8 +12,6 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright Buildbot Team Members
-
-from __future__ import with_statement
 r"""
 Buildbot metrics module
 
@@ -33,19 +31,21 @@ Basic architecture:
           \/
     MetricWatcher
 """
+
+import gc
+import os
+import sys
+from collections import defaultdict
 from collections import deque
 
-from buildbot import config
-from buildbot import util
-from collections import defaultdict
 from twisted.application import service
 from twisted.internet import reactor
 from twisted.internet.task import LoopingCall
 from twisted.python import log
 
-import gc
-import os
-import sys
+from buildbot import util
+from buildbot.util import service as util_service
+
 # Make use of the resource module if we can
 try:
     import resource
@@ -54,7 +54,7 @@ except ImportError:
     resource = None
 
 
-class MetricEvent(object):
+class MetricEvent:
 
     @classmethod
     def log(cls, *args, **kwargs):
@@ -75,7 +75,8 @@ class MetricTimeEvent(MetricEvent):
         self.timer = timer
         self.elapsed = elapsed
 
-ALARM_OK, ALARM_WARN, ALARM_CRIT = range(3)
+
+ALARM_OK, ALARM_WARN, ALARM_CRIT = list(range(3))
 ALARM_TEXT = ["OK", "WARN", "CRIT"]
 
 
@@ -96,7 +97,7 @@ def countMethod(counter):
     return decorator
 
 
-class Timer(object):
+class Timer:
     # For testing
     _reactor = None
 
@@ -147,7 +148,7 @@ class FiniteList(deque):
 
     def __init__(self, maxlen=10):
         self._maxlen = maxlen
-        deque.__init__(self)
+        super().__init__()
 
     def append(self, o):
         deque.append(self, o)
@@ -158,15 +159,15 @@ class FiniteList(deque):
 class AveragingFiniteList(FiniteList):
 
     def __init__(self, maxlen=10):
-        FiniteList.__init__(self, maxlen)
+        super().__init__(maxlen)
         self.average = 0
 
     def append(self, o):
-        FiniteList.append(self, o)
+        super().append(o)
         self._calc()
 
     def _calc(self):
-        if len(self) == 0:
+        if not self:
             self.average = 0
         else:
             self.average = float(sum(self)) / len(self)
@@ -174,7 +175,7 @@ class AveragingFiniteList(FiniteList):
         return self.average
 
 
-class MetricHandler(object):
+class MetricHandler:
 
     def __init__(self, metrics):
         self.metrics = metrics
@@ -221,7 +222,7 @@ class MetricCountHandler(MetricHandler):
             self._counters[metric.counter] += metric.count
 
     def keys(self):
-        return self._counters.keys()
+        return list(self._counters)
 
     def get(self, counter):
         return self._counters[counter]
@@ -229,14 +230,14 @@ class MetricCountHandler(MetricHandler):
     def report(self):
         retval = []
         for counter in sorted(self.keys()):
-            retval.append("Counter %s: %i" % (counter, self.get(counter)))
+            retval.append(f"Counter {counter}: {self.get(counter)}")
         return "\n".join(retval)
 
     def asDict(self):
         retval = {}
         for counter in sorted(self.keys()):
             retval[counter] = self.get(counter)
-        return dict(counters=retval)
+        return {"counters": retval}
 
 
 class MetricTimeHandler(MetricHandler):
@@ -249,7 +250,7 @@ class MetricTimeHandler(MetricHandler):
         self._timers[metric.timer].append(metric.elapsed)
 
     def keys(self):
-        return self._timers.keys()
+        return list(self._timers)
 
     def get(self, timer):
         return self._timers[timer].average
@@ -257,14 +258,14 @@ class MetricTimeHandler(MetricHandler):
     def report(self):
         retval = []
         for timer in sorted(self.keys()):
-            retval.append("Timer %s: %.3g" % (timer, self.get(timer)))
+            retval.append(f"Timer {timer}: {self.get(timer):.3g}")
         return "\n".join(retval)
 
     def asDict(self):
         retval = {}
         for timer in sorted(self.keys()):
             retval[timer] = self.get(timer)
-        return dict(timers=retval)
+        return {"timers": retval}
 
 
 class MetricAlarmHandler(MetricHandler):
@@ -280,86 +281,53 @@ class MetricAlarmHandler(MetricHandler):
         retval = []
         for alarm, (level, msg) in sorted(self._alarms.items()):
             if msg:
-                retval.append("%s %s: %s" % (ALARM_TEXT[level], alarm, msg))
+                retval.append(f"{ALARM_TEXT[level]} {alarm}: {msg}")
             else:
-                retval.append("%s %s" % (ALARM_TEXT[level], alarm))
+                retval.append(f"{ALARM_TEXT[level]} {alarm}")
         return "\n".join(retval)
 
     def asDict(self):
         retval = {}
         for alarm, (level, msg) in sorted(self._alarms.items()):
             retval[alarm] = (ALARM_TEXT[level], msg)
-        return dict(alarms=retval)
+        return {"alarms": retval}
 
 
-class PollerWatcher(object):
+class AttachedWorkersWatcher:
 
     def __init__(self, metrics):
         self.metrics = metrics
 
     def run(self):
-        # Check if 'BuildMaster.pollDatabaseChanges()' and
-        # 'BuildMaster.pollDatabaseBuildRequests()' are running fast enough
-        h = self.metrics.getHandler(MetricTimeEvent)
-        if not h:
-            log.msg("Couldn't get MetricTimeEvent handler")
-            MetricAlarmEvent.log('PollerWatcher',
-                                 msg="Coudln't get MetricTimeEvent handler",
-                                 level=ALARM_WARN)
-            return
-
-        for method in ('BuildMaster.pollDatabaseChanges()',
-                       'BuildMaster.pollDatabaseBuildRequests()'):
-            t = h.get(method)
-            master = self.metrics.parent
-            db_poll_interval = master.config.db['db_poll_interval']
-
-            if db_poll_interval:
-                if t < 0.8 * db_poll_interval:
-                    level = ALARM_OK
-                elif t < db_poll_interval:
-                    level = ALARM_WARN
-                else:
-                    level = ALARM_CRIT
-                MetricAlarmEvent.log(method, level=level)
-
-
-class AttachedSlavesWatcher(object):
-
-    def __init__(self, metrics):
-        self.metrics = metrics
-
-    def run(self):
-        # Check if 'BotMaster.attached_slaves' equals
-        # 'AbstractBuildSlave.attached_slaves'
+        # Check if 'BotMaster.attached_workers' equals
+        # 'AbstractWorker.attached_workers'
         h = self.metrics.getHandler(MetricCountEvent)
         if not h:
             log.msg("Couldn't get MetricCountEvent handler")
-            MetricAlarmEvent.log('AttachedSlavesWatcher',
+            MetricAlarmEvent.log('AttachedWorkersWatcher',
                                  msg="Coudln't get MetricCountEvent handler",
                                  level=ALARM_WARN)
             return
-        botmaster_count = h.get('BotMaster.attached_slaves')
-        buildslave_count = h.get('AbstractBuildSlave.attached_slaves')
+        botmaster_count = h.get('BotMaster.attached_workers')
+        worker_count = h.get('AbstractWorker.attached_workers')
 
         # We let these be off by one since they're counted at slightly
         # different times
-        if abs(botmaster_count - buildslave_count) > 1:
+        if abs(botmaster_count - worker_count) > 1:
             level = ALARM_WARN
         else:
             level = ALARM_OK
 
-        MetricAlarmEvent.log('attached_slaves',
-                             msg='%s %s' % (botmaster_count, buildslave_count),
+        MetricAlarmEvent.log('attached_workers', msg=f'{botmaster_count} {worker_count}',
                              level=level)
 
 
 def _get_rss():
-    if sys.platform == 'linux2':
+    if sys.platform == 'linux':
         try:
-            with open("/proc/%i/statm" % os.getpid()) as f:
+            with open(f"/proc/{os.getpid()}/statm", encoding='utf-8') as f:
                 return int(f.read().split()[1])
-        except:
+        except Exception:
             return 0
     return 0
 
@@ -387,8 +355,9 @@ def periodicCheck(_reactor=reactor):
                 v = r[i]
                 if a == 'ru_maxrss' and v == 0:
                     v = _get_rss() * resource.getpagesize() / 1024
-                MetricCountEvent.log('resource.%s' % a, v, absolute=True)
-            MetricCountEvent.log('resource.pagesize', resource.getpagesize(), absolute=True)
+                MetricCountEvent.log(f'resource.{a}', v, absolute=True)
+            MetricCountEvent.log(
+                'resource.pagesize', resource.getpagesize(), absolute=True)
         # Measure the reactor delay
         then = util.now(_reactor)
         dt = 0.1
@@ -402,12 +371,12 @@ def periodicCheck(_reactor=reactor):
         log.err(None, "while collecting VM metrics")
 
 
-class MetricLogObserver(config.ReconfigurableServiceMixin,
+class MetricLogObserver(util_service.ReconfigurableServiceMixin,
                         service.MultiService):
     _reactor = reactor
 
     def __init__(self):
-        service.MultiService.__init__(self)
+        super().__init__()
         self.setName('metrics')
 
         self.enabled = False
@@ -424,12 +393,10 @@ class MetricLogObserver(config.ReconfigurableServiceMixin,
         self.registerHandler(MetricTimeEvent, MetricTimeHandler(self))
         self.registerHandler(MetricAlarmEvent, MetricAlarmHandler(self))
 
-        # Make sure our changes poller is behaving
-        self.getHandler(MetricTimeEvent).addWatcher(PollerWatcher(self))
         self.getHandler(MetricCountEvent).addWatcher(
-            AttachedSlavesWatcher(self))
+            AttachedWorkersWatcher(self))
 
-    def reconfigService(self, new_config):
+    def reconfigServiceWithBuildbotConfig(self, new_config):
         # first, enable or disable
         if new_config.metrics is None:
             self.disable()
@@ -462,12 +429,11 @@ class MetricLogObserver(config.ReconfigurableServiceMixin,
                     self.periodic_task.start(periodic_interval)
 
         # upcall
-        return config.ReconfigurableServiceMixin.reconfigService(self,
-                                                                 new_config)
+        return super().reconfigServiceWithBuildbotConfig(new_config)
 
     def stopService(self):
         self.disable()
-        service.MultiService.stopService(self)
+        super().stopService()
 
     def enable(self):
         if self.enabled:
@@ -514,17 +480,17 @@ class MetricLogObserver(config.ReconfigurableServiceMixin,
 
     def asDict(self):
         retval = {}
-        for interface, handler in self.handlers.iteritems():
+        for _, handler in self.handlers.items():
             retval.update(handler.asDict())
         return retval
 
     def report(self):
         try:
-            for interface, handler in self.handlers.iteritems():
+            for handler in self.handlers.values():
                 report = handler.report()
                 if not report:
                     continue
                 for line in report.split("\n"):
                     log.msg(line)
-        except:
+        except Exception:
             log.err(None, "generating metric report")
