@@ -13,14 +13,17 @@
 #
 # Copyright Buildbot Team Members
 
+
 import os
 
 from twisted.internet import defer
 from twisted.internet import reactor
 from twisted.python import log
 
-from buildbot.interfaces import BuildSlaveTooOldError
+from buildbot.interfaces import WorkerSetupError
 from buildbot.process import buildstep
+from buildbot.process import remotecommand
+from buildbot.process import results
 from buildbot.steps.source.base import Source
 
 
@@ -37,64 +40,54 @@ class Bzr(Source):
         self.branch = defaultBranch
         self.mode = mode
         self.method = method
-        Source.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         if repourl and baseURL:
             raise ValueError("you must provide exactly one of repourl and"
                              " baseURL")
 
         if repourl is None and baseURL is None:
-            raise ValueError("you must privide at least one of repourl and"
+            raise ValueError("you must provide at least one of repourl and"
                              " baseURL")
 
         if baseURL is not None and defaultBranch is None:
             raise ValueError("you must provide defaultBranch with baseURL")
 
-        assert self.mode in ['incremental', 'full']
+        if not self._hasAttrGroupMember('mode', self.mode):
+            raise ValueError(f"mode {self.mode} is not one of {self._listAttrGroupMembers('mode')}")
 
         if self.mode == 'full':
             assert self.method in ['clean', 'fresh', 'clobber', 'copy', None]
 
-    def startVC(self, branch, revision, patch):
+    @defer.inlineCallbacks
+    def run_vc(self, branch, revision, patch):
         if branch:
             self.branch = branch
         self.revision = revision
         self.method = self._getMethod()
-        self.stdio_log = self.addLogForRemoteCommands("stdio")
+        self.stdio_log = yield self.addLogForRemoteCommands("stdio")
 
         if self.repourl is None:
             self.repourl = os.path.join(self.baseURL, self.branch)
 
-        d = self.checkBzr()
+        installed = yield self.checkBzr()
 
-        def checkInstall(bzrInstalled):
-            if not bzrInstalled:
-                raise BuildSlaveTooOldError("bzr is not installed on slave")
-            return 0
-        d.addCallback(checkInstall)
+        if not installed:
+            raise WorkerSetupError("bzr is not installed on worker")
 
-        d.addCallback(lambda _: self.sourcedirIsPatched())
+        patched = yield self.sourcedirIsPatched()
 
-        def checkPatched(patched):
-            if patched:
-                return self._dovccmd(['clean-tree', '--ignored', '--force'])
-            else:
-                return 0
-        d.addCallback(checkPatched)
+        if patched:
+            yield self._dovccmd(['clean-tree', '--ignored', '--force'])
 
-        if self.mode == 'full':
-            d.addCallback(lambda _: self.full())
-        elif self.mode == 'incremental':
-            d.addCallback(lambda _: self.incremental())
+        yield self._getAttrGroupMember('mode', self.mode)()
 
         if patch:
-            d.addCallback(self.patch, patch)
-        d.addCallback(self.parseGotRevision)
-        d.addCallback(self.finish)
-        d.addErrback(self.failed)
-        return d
+            yield self.patch(patch)
+        yield self.parseGotRevision()
+        return results.SUCCESS
 
     @defer.inlineCallbacks
-    def incremental(self):
+    def mode_incremental(self):
         updatable = yield self._sourcedirIsUpdatable()
         if updatable:
             command = ['update']
@@ -105,7 +98,7 @@ class Bzr(Source):
             yield self._doFull()
 
     @defer.inlineCallbacks
-    def full(self):
+    def mode_full(self):
         if self.method == 'clobber':
             yield self.clobber()
             return
@@ -125,41 +118,35 @@ class Bzr(Source):
         else:
             raise ValueError("Unknown method, check your configuration")
 
+    @defer.inlineCallbacks
     def _clobber(self):
-        cmd = buildstep.RemoteCommand('rmdir', {'dir': self.workdir,
-                                                'logEnviron': self.logEnviron, })
+        cmd = remotecommand.RemoteCommand('rmdir', {'dir': self.workdir,
+                                                    'logEnviron': self.logEnviron, })
         cmd.useLog(self.stdio_log, False)
-        d = self.runCommand(cmd)
+        yield self.runCommand(cmd)
 
-        def checkRemoval(res):
-            if res != 0:
-                raise RuntimeError("Failed to delete directory")
-            return res
-        d.addCallback(lambda _: checkRemoval(cmd.rc))
-        return d
+        if cmd.rc != 0:
+            raise RuntimeError("Failed to delete directory")
 
+    @defer.inlineCallbacks
     def clobber(self):
-        d = self._clobber()
-        d.addCallback(lambda _: self._doFull())
-        return d
+        yield self._clobber()
+        yield self._doFull()
 
+    @defer.inlineCallbacks
     def copy(self):
-        cmd = buildstep.RemoteCommand('rmdir', {'dir': 'build',
-                                                'logEnviron': self.logEnviron, })
+        cmd = remotecommand.RemoteCommand('rmdir', {'dir': 'build',
+                                                    'logEnviron': self.logEnviron, })
         cmd.useLog(self.stdio_log, False)
-        d = self.runCommand(cmd)
-        d.addCallback(lambda _: self.incremental())
+        yield self.runCommand(cmd)
+        yield self.mode_incremental()
 
-        def copy(_):
-            cmd = buildstep.RemoteCommand('cpdir',
+        cmd = remotecommand.RemoteCommand('cpdir',
                                           {'fromdir': 'source',
                                            'todir': 'build',
                                            'logEnviron': self.logEnviron, })
-            cmd.useLog(self.stdio_log, False)
-            d = self.runCommand(cmd)
-            return d
-        d.addCallback(copy)
-        return d
+        cmd.useLog(self.stdio_log, False)
+        yield self.runCommand(cmd)
 
     def clean(self):
         d = self._dovccmd(['clean-tree', '--ignored', '--force'])
@@ -177,47 +164,33 @@ class Bzr(Source):
         d.addCallback(lambda _: self._dovccmd(command))
         return d
 
+    @defer.inlineCallbacks
     def _doFull(self):
         command = ['checkout', self.repourl, '.']
         if self.revision:
             command.extend(['-r', self.revision])
 
         if self.retry:
-            abandonOnFailure = (self.retry[1] <= 0)
+            abandonOnFailure = self.retry[1] <= 0
         else:
             abandonOnFailure = True
-        d = self._dovccmd(command, abandonOnFailure=abandonOnFailure)
 
-        def _retry(res):
+        res = yield self._dovccmd(command, abandonOnFailure=abandonOnFailure)
+
+        if self.retry:
             if self.stopped or res == 0:
                 return res
             delay, repeats = self.retry
             if repeats > 0:
-                log.msg("Checkout failed, trying %d more times after %d seconds"
-                        % (repeats, delay))
+                log.msg(f"Checkout failed, trying {repeats} more times after {delay} seconds")
                 self.retry = (delay, repeats - 1)
                 df = defer.Deferred()
                 df.addCallback(lambda _: self._clobber())
                 df.addCallback(lambda _: self._doFull())
                 reactor.callLater(delay, df.callback, None)
-                return df
-            return res
+                res = yield df
 
-        if self.retry:
-            d.addCallback(_retry)
-        return d
-
-    def finish(self, res):
-        d = defer.succeed(res)
-
-        def _gotResults(results):
-            self.setStatus(self.cmd, results)
-            log.msg("Closing log, sending result of the command %s " %
-                    (self.cmd))
-            return results
-        d.addCallback(_gotResults)
-        d.addCallback(self.finished)
-        return d
+        return res
 
     def _sourcedirIsUpdatable(self):
         return self.pathExists(self.build.path_module.join(self.workdir, '.bzr'))
@@ -225,37 +198,34 @@ class Bzr(Source):
     def computeSourceRevision(self, changes):
         if not changes:
             return None
-        lastChange = max([int(c.revision) for c in changes])
+        lastChange = max(int(c.revision) for c in changes)
         return lastChange
 
     def _dovccmd(self, command, abandonOnFailure=True, collectStdout=False):
-        cmd = buildstep.RemoteShellCommand(self.workdir, ['bzr'] + command,
-                                           env=self.env,
-                                           logEnviron=self.logEnviron,
-                                           timeout=self.timeout,
-                                           collectStdout=collectStdout)
+        cmd = remotecommand.RemoteShellCommand(self.workdir, ['bzr'] + command,
+                                               env=self.env,
+                                               logEnviron=self.logEnviron,
+                                               timeout=self.timeout,
+                                               collectStdout=collectStdout)
         cmd.useLog(self.stdio_log, False)
         d = self.runCommand(cmd)
 
-        def evaluateCommand(cmd):
+        @d.addCallback
+        def evaluateCommand(_):
             if abandonOnFailure and cmd.didFail():
-                log.msg("Source step failed while running command %s" % cmd)
+                log.msg(f"Source step failed while running command {cmd}")
                 raise buildstep.BuildStepFailed()
             if collectStdout:
                 return cmd.stdout
-            else:
-                return cmd.rc
-        d.addCallback(lambda _: evaluateCommand(cmd))
+            return cmd.rc
         return d
 
     def checkBzr(self):
         d = self._dovccmd(['--version'])
 
+        @d.addCallback
         def check(res):
-            if res == 0:
-                return True
-            return False
-        d.addCallback(check)
+            return res == 0
         return d
 
     def _getMethod(self):
@@ -265,21 +235,19 @@ class Bzr(Source):
             return None
         elif self.method is None and self.mode == 'full':
             return 'fresh'
+        return None
 
-    def parseGotRevision(self, _):
-        d = self._dovccmd(["version-info", "--custom", "--template='{revno}"],
-                          collectStdout=True)
+    @defer.inlineCallbacks
+    def parseGotRevision(self):
+        stdout = yield self._dovccmd(["version-info", "--custom", "--template='{revno}"],
+                                     collectStdout=True)
 
-        def setrev(stdout):
-            revision = stdout.strip("'")
-            try:
-                int(revision)
-            except ValueError:
-                log.msg("Invalid revision number")
-                raise buildstep.BuildStepFailed()
+        revision = stdout.strip("'")
+        try:
+            int(revision)
+        except ValueError as e:
+            log.msg("Invalid revision number")
+            raise buildstep.BuildStepFailed() from e
 
-            log.msg("Got Git revision %s" % (revision, ))
-            self.updateSourceProperty('got_revision', revision)
-            return 0
-        d.addCallback(setrev)
-        return d
+        log.msg(f"Got Git revision {revision}")
+        self.updateSourceProperty('got_revision', revision)
