@@ -15,8 +15,7 @@
 
 
 import os
-
-import mock
+from unittest import mock
 
 from twisted.internet import defer
 from twisted.internet import reactor
@@ -96,9 +95,9 @@ class Schedulers(RunMasterBase, www.RequiresWwwMixin):
     def setup_config(self, extra_config):
         c = {}
         from buildbot.config import BuilderConfig
+        from buildbot.process import results
         from buildbot.process.buildstep import BuildStep
         from buildbot.process.factory import BuildFactory
-        from buildbot.process import results
 
         class MyBuildStep(BuildStep):
 
